@@ -1,3 +1,7 @@
+import re
+
+from docstring_parser import parse as _doc_parse
+
 from .modules.aritmeticlogic import AritmeticLogic
 from .modules.codetidy import CodeTidy
 from .modules.compression import Compression
@@ -8,16 +12,18 @@ from .modules.extractors import Extractors
 from .modules.forensics import Forensics
 from .modules.hashing import Hashing
 from .modules.language import Language
+from .modules.links import Links
 from .modules.multimedia import Multimedia
 from .modules.networking import Networking
 from .modules.other import Other
-from .modules.pcap import Pcap
 from .modules.publickey import Publickey
+from .modules.search import Search
 from .modules.utils import Utils
+from .modules.internal.colors import cyan
 
-from .conf import Config
+from .config import ChepyConfig
 
-_plugins = Config().load_plugins()
+_plugins = ChepyConfig().load_plugins()
 
 
 class Chepy(
@@ -31,12 +37,26 @@ class Chepy(
     Forensics,
     Hashing,
     Language,
+    Links,
     Multimedia,
     Networking,
     Other,
-    Pcap,
     Publickey,
+    Search,
     Utils,
     *_plugins
 ):
     pass
+
+
+def search_chepy_methods(search: str) -> None:  # pragma: no cover
+    """Search for Chepy methods
+    
+    Args:
+        search (str): String to search for
+    """
+    methods = dir(Chepy)
+    for method in methods:
+        if search in method and not method.startswith("_"):
+            docs = _doc_parse(getattr(Chepy, method).__doc__).short_description
+            print(cyan(method), docs)
