@@ -21,15 +21,17 @@ from prompt_toolkit import PromptSession
 from chepy import Chepy
 from chepy.__version__ import __version__
 import chepy.modules.internal.cli as chepy_cli
-from chepy.modules.internal.colors import RED, YELLOW, CYAN
-from chepy.conf import Config
+from chepy.modules.internal.colors import red, yellow, cyan, magenta, green
+from chepy.config import ChepyConfig
 
-config = Config()
+config = ChepyConfig()
 options = []
 chepy = dir(Chepy)
 fire_obj = None
 errors = []
 
+prompt_colors = config.prompt_colors.split()
+
 
 def get_style():
     return Style.from_dict(
@@ -37,18 +39,18 @@ def get_style():
             "completion-menu.completion.current": "bg:#00aaaa #000000",
             # "completion-menu.completion": "bg:#008888 #ffffff",
             "completion-menu.completion.fuzzymatch.outside": "fg:#00aaaa",
-            "prompt1": "#00ffff bold",
-            "prompt2": "#ff0000 bold",
-            "prompt3": "#ffd700 bold",
+            "prompt1": "{} bold".format(prompt_colors[0]),
+            "prompt2": "{} bold".format(prompt_colors[1]),
+            "prompt3": "{} bold".format(prompt_colors[2]),
             "state_index": "#ffd700",
-            "file": "#00ff48",
-            "rprompt": "fg:#00ff48",
-            "bottom-toolbar": "#000000",
-            "bt_version": "bg:#00ff48",
-            "bt_states": "bg:#60cdd5",
-            "bt_buffers": "bg:#ff00ff",
-            "bt_type": "bg:#ffd700",
-            "bt_errors": "bg:#ff0000",
+            "rprompt": "fg:{}".format(config.prompt_rprompt),
+            "bottom-toolbar": config.prompt_bottom_toolbar,
+            "prompt_toolbar_version": "bg:{}".format(config.prompt_toolbar_version),
+            "prompt_toolbar_states": "bg:{}".format(config.prompt_toolbar_states),
+            "prompt_toolbar_buffers": "bg:{}".format(config.prompt_toolbar_buffers),
+            "prompt_toolbar_type": "bg:{}".format(config.prompt_toolbar_type),
+            "prompt_toolbar_plugins": "bg:{}".format(config.prompt_toolbar_plugins),
+            "prompt_toolbar_errors": "bg:{}".format(config.prompt_toolbar_errors),
         }
     )
 
@@ -92,7 +94,11 @@ def get_options():
 
 
 def prompt_message(fire_obj):
-    elements = [("class:prompt1", ">"), ("class:prompt2", ">"), ("class:prompt3", ">")]
+    elements = [
+        ("class:prompt1", config.prompt_char),
+        ("class:prompt2", config.prompt_char),
+        ("class:prompt3", config.prompt_char),
+    ]
     try:
         elements.append(
             (
@@ -116,23 +122,33 @@ def bottom_toolbar(fire_obj):
         current_state = fire_obj._current_index if fire_obj is not None else 0
         buffers = len(fire_obj.buffers) if fire_obj is not None else 0
         return [
-            ("class:bt_version", "Chepy: {} ".format(__version__)),
+            ("class:prompt_toolbar_version", "Chepy: {} ".format(__version__)),
             (
-                "class:bt_states",
+                "class:prompt_toolbar_states",
                 " States: {current}/{total} ".format(
                     current=current_state, total=states
                 ),
             ),
-            ("class:bt_buffers", " Buffers: {total} ".format(total=buffers)),
-            ("class:bt_type", " State: {} ".format(type(fire_obj.state).__name__)),
-            ("class:bt_errors", " Errors: {} ".format(len(errors))),
+            (
+                "class:prompt_toolbar_buffers",
+                " Buffers: {total} ".format(total=buffers),
+            ),
+            (
+                "class:prompt_toolbar_type",
+                " State: {} ".format(type(fire_obj.state).__name__),
+            ),
+            (
+                "class:prompt_toolbar_plugins",
+                " Plugins: {} ".format(config.enable_plugins),
+            ),
+            ("class:prompt_toolbar_errors", " Errors: {} ".format(len(errors))),
         ]
 
 
 class CustomValidator(Validator):
     def validate(self, document):
         text = document.text.split()
-        if document.text.startswith("!"):
+        if re.search(r"^(!|#|\?)", document.text):
             pass
         elif len(text) > 1:
             if not text[-2].startswith("--"):
@@ -184,8 +200,10 @@ class CustomCompleter(Completer):
                 )
                 not_chepy_obj = ""
                 if method_docs.get("returns"):
-                    if method_docs["returns"] != "Chepy":
-                        not_chepy_obj = "bg:#ffd700"
+                    if method_docs["returns"] == None:
+                        not_chepy_obj = "bg:{}".format(config.prompt_cli_method)
+                    elif method_docs["returns"] == "ChepyPlugin":
+                        not_chepy_obj = "bg:{}".format(config.prompt_plugin_method)
                 yield Completion(
                     method_name,
                     start_position=-len(word),
@@ -195,10 +213,13 @@ class CustomCompleter(Completer):
 
 
 def get_current_type(obj):
-    if obj:
-        return type(obj).__name__
+    if config.show_rprompt:
+        if obj:
+            return type(obj).__name__
+        else:
+            return "Type of current state"
     else:
-        return "Type of current state"
+        return None
 
 
 def parse_args(args):
@@ -206,6 +227,7 @@ def parse_args(args):
     parse.add_argument(
         "-v", "--version", action="version", version="%(prog)s " + __version__
     )
+    parse.add_argument("-r", "--recipe", dest="recipe")
     parse.add_argument("data", nargs="*")
     return parse.parse_args(args)
 
@@ -217,69 +239,89 @@ def main():
     args = parse_args(sys.argv[1:])
     args_data = args.data
 
-    args_data.append("-")
+    if args.recipe:
+        print(Chepy(*args_data).load_recipe(args.recipe).o)
+    else:
+        args_data.append("-")
 
-    history_file = config.history_path
-    session = PromptSession(
-        history=FileHistory(history_file),
-        style=get_style(),
-        wrap_lines=True,
-        auto_suggest=AutoSuggestFromHistory(),
-    )
-    try:
-        while True:
-            prompt = session.prompt(
-                prompt_message(fire_obj=fire_obj),
-                bottom_toolbar=bottom_toolbar(fire_obj),
-                completer=FuzzyCompleter(
-                    merge_completers([CustomCompleter(), chepy_cli.CliCompleter()])
-                ),
-                validator=CustomValidator(),
-                rprompt=get_current_type(fire_obj),
-            )
+        history_file = config.history_path
+        session = PromptSession(
+            history=FileHistory(history_file),
+            style=get_style(),
+            wrap_lines=True,
+            auto_suggest=AutoSuggestFromHistory(),
+        )
+        try:
+            while True:
+                prompt = session.prompt(
+                    prompt_message(fire_obj=fire_obj),
+                    bottom_toolbar=bottom_toolbar(fire_obj),
+                    completer=FuzzyCompleter(
+                        merge_completers([CustomCompleter(), chepy_cli.CliCompleter()])
+                    ),
+                    validator=CustomValidator(),
+                    rprompt=get_current_type(fire_obj),
+                )
 
-            # check and output any commands that start with cli_
-            if re.match(r"^\!", prompt):
-                print(subprocess.getoutput(re.sub(r"^\!\s?", "", prompt)))
-            elif re.search(r"^cli_.+", prompt):
-                cli_method = prompt.split()[0]
-                cli_args = re.search(r"--(\w+)\s(\w+)", prompt)
-                if cli_method == "cli_show_errors":
-                    getattr(chepy_cli, "cli_show_errors")(errors)
-                elif cli_method == "cli_go_back":
-                    args_data = args_data[: -len(last_command + ["-"])]
-                    print(CYAN("Go back: {}".format(last_command)))
-                elif cli_method == "cli_delete_history":
-                    Path(config.history_path).unlink()
-                elif cli_args:
-                    getattr(chepy_cli, cli_method)(
-                        fire_obj, **{cli_args.group(1): cli_args.group(2)}
-                    )
-                else:
-                    getattr(chepy_cli, cli_method)(fire_obj)
-
-            else:
-                for method in chepy:
-                    if not method.startswith("_") and not isinstance(
-                        getattr(Chepy, method), property
-                    ):
-                        fire.decorators._SetMetadata(
-                            getattr(Chepy, method),
-                            fire.decorators.ACCEPTS_POSITIONAL_ARGS,
-                            False,
+                # check and output any commands that start with cli_
+                if re.match(r"^\!", prompt):
+                    print(magenta(subprocess.getoutput(re.sub(r"^\!\s?", "", prompt))))
+                # check if line is a comment
+                elif re.match(r"^#", prompt):
+                    print(cyan(prompt))
+                # get help for a method
+                elif re.match(r"^\?", prompt):
+                    _method_name = re.match(r"^\?(\s?)+([\w_]+)", prompt).group(2)
+                    chepy_cli.get_doc(_method_name)
+                # check if method called is a cli method
+                elif re.search(r"^cli_.+", prompt):
+                    cli_method = prompt.split()[0]
+                    cli_args = re.search(r"--(\w+)\s([\w\W]+)", prompt)
+                    if cli_method == "cli_show_errors":
+                        getattr(chepy_cli, "cli_show_errors")(errors)
+                    elif cli_method == "cli_plugin_path":
+                        getattr(chepy_cli, "cli_plugin_path")(config)
+                    elif cli_method == "cli_go_back":
+                        args_data = args_data[: -len(last_command + ["-"])]
+                        print(cyan("Go back: {}".format(last_command)))
+                    elif cli_method == "cli_delete_history":
+                        Path(config.history_path).unlink()
+                    elif cli_args:
+                        getattr(chepy_cli, cli_method)(
+                            fire_obj, **{cli_args.group(1): cli_args.group(2)}
                         )
-                args_data += prompt.split()
-                if args_data[-1] != "-":
-                    args_data.append("-")
-                try:
-                    last_command = prompt.split() + ["-"]
-                    fire_obj = fire.Fire(Chepy, command=args_data)
-                except:
-                    # go back to last working arg
-                    e_type, e_msg, e_traceback = sys.exc_info()
-                    print(RED(e_type.__name__), YELLOW(e_msg.__str__()))
-                    args_data = args_data[: -len(last_command)]
-                    continue
-    except KeyboardInterrupt:
-        print("OKBye")
-        sys.exit()
+                    else:
+                        getattr(chepy_cli, cli_method)(fire_obj)
+
+                else:
+                    for method in chepy:
+                        if not method.startswith("_") and not isinstance(
+                            getattr(Chepy, method), property
+                        ):
+                            fire.decorators._SetMetadata(
+                                getattr(Chepy, method),
+                                fire.decorators.ACCEPTS_POSITIONAL_ARGS,
+                                False,
+                            )
+                    args_data += prompt.split()
+                    if args_data[-1] != "-":
+                        args_data.append("-")
+                    try:
+                        last_command = prompt.split() + ["-"]
+                        fire_obj = fire.Fire(Chepy, command=args_data)
+                    # handle required args for methods
+                    except fire.core.FireExit:
+                        args_data = args_data[: -len(last_command)]
+                    except TypeError as e:
+                        print(red(e.message))
+                    except SystemExit:
+                        sys.exit()
+                    except:
+                        # go back to last working arg
+                        e_type, e_msg, e_traceback = sys.exc_info()
+                        print(red(e_type.__name__), yellow(e_msg.__str__()))
+                        args_data = args_data[: -len(last_command)]
+                        continue
+        except KeyboardInterrupt:
+            print(green("\nOKBye"))
+            sys.exit()
