@@ -1,3 +1,4 @@
+import sys
 import base64
 import binascii
 import pathlib
@@ -6,18 +7,19 @@ import logging
 import inspect
 import io
 import itertools
+import subprocess
+from configparser import ConfigParser
+from urllib.parse import urljoin
+from pprint import pformat
 from typing import Any, Tuple, List, Union
 
 import pyperclip
-import requests
 import ujson
 import jsonpickle
 import regex as re
-import scapy.all as scapy
 from decorator import decorator
 
-from .modules.exceptions import PrintException
-from .modules.internal.colors import YELLOW, CYAN, GREEN, MAGENTA
+from .modules.internal.colors import yellow, cyan, green, magenta, blue, red
 
 
 class ChepyDecorators(object):
@@ -79,10 +81,10 @@ class ChepyCore(object):
         self.bake = self.out
         #: Alias for `web`
         self.cyberchef = self.web
+        #: Alias for `load_file`
+        self.read_file = self.load_file
         #: Holds all the methods that are called/chanined and their args
         self._stack = list()
-        #: Holder for scapy pcap reader
-        self._pcap_filepath = None
 
         #: Log level
         self.log_level = logging.INFO
@@ -102,7 +104,7 @@ class ChepyCore(object):
     def __str__(self):
         try:
             if isinstance(self.state, bytearray):
-                return "bytearray in state"
+                return re.sub(rb"[^\x00-\x7f]", b".", self.state).decode()
             else:
                 return self._convert_to_str()
         except UnicodeDecodeError:  # pragma: no cover
@@ -158,7 +160,7 @@ class ChepyCore(object):
         Returns:
             Chepy: The Chepy object. 
         """
-        logging.info(data)
+        logging.info(blue(data))
         return None
 
     def _warning_logger(self, data: str) -> None:  # pragma: no cover
@@ -170,7 +172,19 @@ class ChepyCore(object):
         Returns:
             Chepy: The Chepy object. 
         """
-        logging.warning(data)
+        logging.warning(yellow(data))
+        return None
+
+    def _error_logger(self, data: str) -> None:  # pragma: no cover
+        """Just a binding for logger.error
+        
+        Args:
+            data (str): Message to log
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        logging.error(red(data))
         return None
 
     def fork(self, methods: List[Tuple[Union[str, object], dict]]):
@@ -593,28 +607,37 @@ class ChepyCore(object):
         return None
 
     @ChepyDecorators.call_stack
-    def web(self, magic: bool = False) -> None:  # pragma: no cover
+    def web(
+        self,
+        magic: bool = False,
+        cyberchef_url: str = "https://gchq.github.io/CyberChef/",
+    ) -> None:  # pragma: no cover
         """Opens the current string in CyberChef on the browser as hex
 
         Args:
             magic (bool, optional): Start with the magic method in CyberChef
+            cyberchef_url (string, optional): Base url for Cyberchef
         
         Returns:
             None: Opens the current data in CyberChef
         """
         data = re.sub(
-            b"=", "", base64.b64encode(binascii.hexlify(self._convert_to_bytes()))
+            b"=+$", "", base64.b64encode(binascii.hexlify(self._convert_to_bytes()))
         )
         if magic:
-            url = "https://gchq.github.io/CyberChef/#recipe=From_Hex('None')Magic(3,false,false,'')&input={}".format(
-                data.decode()
+            url = urljoin(
+                cyberchef_url,
+                "#recipe=From_Hex('None')Magic(3,false,false,'')&input={}".format(
+                    data.decode()
+                ),
             )
         else:
-            url = "https://gchq.github.io/CyberChef/#recipe=From_Hex('None')&input={}".format(
-                data.decode()
+            url = urljoin(
+                cyberchef_url,
+                "#recipe=From_Hex('None')&input={}".format(data.decode()),
             )
         webbrowser.open_new_tab(url)
-        return None
+        return self
 
     @ChepyDecorators.call_stack
     def http_request(
@@ -625,7 +648,7 @@ class ChepyCore(object):
         headers: dict = {},
         cookies: dict = {},
     ):
-        """Get data from http request
+        """Make a http/s request
 
         Make a HTTP/S request and work with the data in Chepy. Most common http 
         methods are supported; but some methods may not provide a response body. 
@@ -672,10 +695,16 @@ class ChepyCore(object):
             else:
                 raise NotImplementedError
 
+        try:
+            from requests import request
+        except ImportError:  # pragma: no cover
+            self._error_logger("Could not import requests. pip install requests")
+            return self
+
         params = json2str(params)
         headers = json2str(headers)
         cookies = json2str(cookies)
-        res = requests.request(
+        res = request(
             method=method,
             url=self.state,
             params=params,
@@ -686,10 +715,73 @@ class ChepyCore(object):
         self.state = {
             "body": res.text,
             "status": res.status_code,
-            "headers": res.headers,
+            "headers": dict(res.headers),
         }
         return self
 
+    @ChepyDecorators.call_stack
+    def load_from_url(
+        self,
+        method: str = "GET",
+        params: dict = {},
+        json: dict = None,
+        headers: dict = {},
+        cookies: dict = {},
+    ):
+        """Load binary content from a url
+
+        Most common http methods are supported; but some methods may not provide a response body. 
+        
+        Args:
+            method (str, optional): Request method. Defaults to 'GET'.
+            params (dict, optional): Query Args. Defaults to {}.
+            json (dict, optional): Request payload. Defaults to None.
+            headers (dict, optional): Headers for request. Defaults to {}.
+            cookies (dict, optional): Cookies for request. Defaults to {}.
+        
+        Raises:
+            NotImplementedError: If state is not a string or dictionary
+            requests.RequestException: If response status code is not 200
+        
+        Returns:
+            Chepy: A bytearray of the response content. The Chepy object.
+
+        Examples:
+            By default, this methed with make a GET request, But supports most 
+            common methods. 
+            
+                >>> c = Chepy("http://example.com/file.png").load_from_url()
+                >>> b'\\x89PNG...'
+        """
+
+        def json2str(obj):  # pragma: no cover
+            if isinstance(obj, dict):
+                return obj
+            elif isinstance(obj, str):
+                return json.loads(obj)
+            else:
+                raise NotImplementedError
+
+        try:
+            from requests import request
+        except ImportError:  # pragma: no cover
+            self._error_logger("Could not import requests. pip install requests")
+            return self
+
+        params = json2str(params)
+        headers = json2str(headers)
+        cookies = json2str(cookies)
+        res = request(
+            method=method,
+            url=self.state,
+            params=params,
+            json=json,
+            headers=headers,
+            cookies=cookies,
+        )
+        self.state = io.BytesIO(res.content).read()
+        return self
+
     @ChepyDecorators.call_stack
     def load_dir(self, pattern: str = "*"):
         """Load all file paths in a directory
@@ -709,7 +801,7 @@ class ChepyCore(object):
 
     @ChepyDecorators.call_stack
     def load_file(self):
-        """If a path is provided, read the file
+        """If a path is provided, load the file
         
         Returns:
             Chepy: The Chepy object. 
@@ -719,7 +811,7 @@ class ChepyCore(object):
             >>> # at the moment, the state only contains the string "/path/to/file"
             >>> c.load_file() # this will load the file content into the state
         """
-        path = pathlib.Path(self.state).expanduser().absolute()
+        path = pathlib.Path(str(self.state)).expanduser().absolute()
         try:
             with open(path, "r") as f:
                 self.states[self._current_index] = f.read()
@@ -728,18 +820,6 @@ class ChepyCore(object):
                 self.states[self._current_index] = bytearray(f.read())
         return self
 
-    @ChepyDecorators.call_stack
-    def read_file(self):
-        """If a path is provided, read the file 
-
-        Alias of `load_file`.
-        
-        Returns:
-            Chepy: The Chepy object. 
-        """
-        self.load_file()
-        return self
-
     def write_to_file(self, path: str, as_binary: bool = False) -> None:
         """Save the state to disk. Return None.
         
@@ -832,6 +912,40 @@ class ChepyCore(object):
                     getattr(self, function)()
         return self
 
+    @ChepyDecorators.call_stack
+    def loop(self, iterations: int, callback: str, args: dict = {}):
+        """Loop and apply callback n times
+        
+        Args:
+            iterations (int): Number of iterations to loop
+            callback (str): The Chepy method to loop over
+            args (dict, optional): Optional arguments for the callback. Defaults to {}.
+        
+        Returns:
+            Chepy: The Chepy object. 
+
+        Examples:
+            >>> c = Chepy("VmpGb2QxTXhXWGxTYmxKV1lrZDRWVmx0ZEV0alZsSllaVWRHYWxWVU1Eaz0=")
+            >>> c.loop(iterations=6, callback='hmac_hash', args={'key': 'secret'})
+            securisec
+        """
+        assert isinstance(callback, str), "Callback must be a string"
+        assert isinstance(iterations, int), "Iterations must be an intiger"
+        assert isinstance(args, dict), "Args must be a dick"
+
+        stack_loop_index = next(
+            itertools.dropwhile(
+                lambda x: self._stack[x]["function"] != "loop",
+                reversed(range(len(self._stack))),
+            )
+        )
+
+        for _ in range(int(iterations)):
+            d = getattr(self, callback)(**args)
+
+        self._stack = self._stack[: stack_loop_index + 1]
+        return self
+
     @ChepyDecorators.call_stack
     def loop_list(self, callback: str, args: dict = {}):
         """Loop over an array and run a Chepy method on it
@@ -852,7 +966,7 @@ class ChepyCore(object):
             ['5cbe6ca2a66b380aec1449d4ebb0d40ac5e1b92e', '30d75bf34740e8781cd4ec7b122e3efd8448e270']
         """
         assert isinstance(self.state, list), "State is not a list"
-        assert isinstance(callback, str), "Callback must be a"
+        assert isinstance(callback, str), "Callback must be a string"
         hold = []
         current_state = self.state
         # find the last index that this method was run
@@ -878,7 +992,8 @@ class ChepyCore(object):
 
     @ChepyDecorators.call_stack
     def loop_dict(self, keys: list, callback: str, args: dict = {}):
-        """Loop over a dictionary and apply the callback to the value
+        """
+        Loop over a dictionary and apply the callback to the value
         
         Args:
             keys (list): List of keys to match
@@ -935,20 +1050,9 @@ class ChepyCore(object):
             self.state = current_state
             raise
 
-    @ChepyDecorators.call_stack
-    def read_pcap(self):
-        """Load a pcap. The state is set to scapy
-        
-        Returns:
-            Chepy: The Chepy object. 
-        """
-        self._pcap_filepath = str(self._abs_path(self.state))
-        self.state = GREEN("Pcap loaded")
-        return self
-
     @ChepyDecorators.call_stack
     def debug(self, verbose: bool = False):
-        """Debug the curren instance of Chepy
+        """Debug the current instance of Chepy
         
         This method does not change the state. 
 
@@ -958,20 +1062,20 @@ class ChepyCore(object):
         Returns:
             Chepy: The Chepy object. 
         """
-        print(CYAN("Current state:"), YELLOW(str(self._current_index)))
-        print(CYAN("Current states:"), YELLOW(str(len(self.states))))
+        print(cyan("Current state:"), yellow(str(self._current_index)))
+        print(cyan("Current states:"), yellow(str(len(self.states))))
         print(
-            CYAN("Current state types:"),
-            YELLOW(str({k: type(v).__name__ for k, v in self.states.items()})),
+            cyan("Current state types:"),
+            yellow(str({k: type(v).__name__ for k, v in self.states.items()})),
         )
-        print(CYAN("Current buffers:"), YELLOW(str(len(self.buffers))))
+        print(cyan("Current buffers:"), yellow(str(len(self.buffers))))
         print(
-            CYAN("Current buffer types:"),
-            YELLOW(str({k: type(v).__name__ for k, v in self.buffers.items()})),
+            cyan("Current buffer types:"),
+            yellow(str({k: type(v).__name__ for k, v in self.buffers.items()})),
         )
         if verbose:
-            print(MAGENTA("States:"), self.states)
-            print(MAGENTA("Buffers:"), self.buffers)
+            print(magenta("States:"), self.states)
+            print(magenta("Buffers:"), self.buffers)
         return self
 
     @ChepyDecorators.call_stack
@@ -984,3 +1088,92 @@ class ChepyCore(object):
         self.states = self.__initial_states
         return self
 
+    @ChepyDecorators.call_stack
+    def shell_output(self):  # pragma: no cover
+        """Run the command in state and get the output
+        
+        Returns:
+            Chepy: The Chepy object. 
+
+        Examples:
+            This method can be used to interace with the shell and Chepy 
+            directly by ingesting a commands output in Chepy. 
+
+            >>> c = Chepy("ls -l").shell_output().o
+            test.html
+            ...
+            test.py
+        """
+        self.state = subprocess.getoutput(self.state)
+        return self
+
+    @ChepyDecorators.call_stack
+    def pretty(self, indent: int = 2): # pragma: no cover
+        """Prettify the state. 
+        
+        Args:
+            indent (int, optional): Indent level. Defaults to 2.
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        self.state = pformat(self.state, indent=int(indent))
+        return self
+
+    def plugins(self, enable: str) -> None:  # pragma: no cover
+        """Use this method to enable or disable Chepy plugins. 
+
+        Valid options are `true` or `false`. Once this method completes, 
+        it does call sys.exit(). 
+
+        Args:
+            enable (str): Set to `true` or `false`
+
+        Returns:
+            None
+        """
+        assert enable in ["true", "false"], "Valid values are true and false"
+        conf_path = pathlib.Path().home() / ".chepy" / "chepy.conf"
+        c = ConfigParser()
+        c.read(conf_path)
+        c.set("Plugins", "enableplugins", enable)
+        with open(conf_path, "w") as f:
+            c.write(f)
+        if enable:
+            self._info_logger(
+                green(
+                    "Plugins have been enabled. Restart Chepy for effects to take place."
+                )
+            )
+        else:
+            self._info_logger(
+                green(
+                    "Plugins have been disabled. Restart Chepy for effects to take place."
+                )
+            )
+        sys.exit()
+        return None
+
+    def set_plugin_path(self, path: str) -> None:  # pragma: no cover
+        """Use this method to set the path for Chepy plugins. 
+
+        Args:
+            path (str): Path to plugins directory
+
+        Returns:
+            None
+        """
+        expand_path = self._abs_path(path)
+        if expand_path.exists():
+            conf_path = pathlib.Path().home() / ".chepy" / "chepy.conf"
+            c = ConfigParser()
+            c.read(conf_path)
+            c.set("Plugins", "pluginpath", str(expand_path))
+            with open(conf_path, "w") as f:
+                c.write(f)
+            self._info_logger(green("Plugin path has been set. Restart for changes."))
+            sys.exit()
+            return None
+        else:
+            raise AttributeError("The path does not exist")
+
