@@ -1,4 +1,3 @@
-import string
 import binascii
 import base64
 import codecs
@@ -7,9 +6,9 @@ import base58
 import ujson
 import yaml
 import regex as re
+import hexdump
 from urllib.parse import quote_plus as _urllib_quote_plus
 from urllib.parse import unquote_plus as _urllib_unquote_plus
-from typing import Any
 
 from ..core import ChepyCore, ChepyDecorators
 from chepy.modules.internal.constants import Encoding
@@ -245,6 +244,30 @@ class DataFormat(ChepyCore):
         self.state = int(self.state)
         return self
 
+    @ChepyDecorators.call_stack
+    def to_bytes(self):
+        """Converts the data in state to bytes
+        
+        Returns:
+            Chepy: The Chepy object.
+
+        Examples:
+            >>> Chepy({'some': 'val', 'kl': 1}).to_bytes().o
+            b"{'some': 'val', 'kl': 1}"
+        """
+        self.state = self._convert_to_str().encode()
+        return self
+
+    @ChepyDecorators.call_stack
+    def from_bytes(self):
+        """Decodes bytes to string.
+        
+        Returns:
+            Chepy: The Chepy object.
+        """
+        self.state = self._convert_to_bytes().decode()
+        return self
+
     @ChepyDecorators.call_stack
     def base64_encode(self, custom: str = None):
         """Encode as Base64
@@ -269,7 +292,7 @@ class DataFormat(ChepyCore):
         if custom is not None:
             x = base64.b64encode(self._convert_to_bytes())
             std_base64chars = (
-                "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
+                "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
             )
             self.state = bytes(
                 str(x)[2:-1].translate(str(x)[2:-1].maketrans(std_base64chars, custom)),
@@ -303,12 +326,18 @@ class DataFormat(ChepyCore):
         """
         if custom is not None:
             std_base64chars = (
-                "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
+                "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
             )
             c = self._convert_to_str().translate(str.maketrans(custom, std_base64chars))
             self.state = base64.b64decode(c.encode())
         else:
-            self.state = base64.b64decode(self._convert_to_bytes())
+            try:
+                self.state = base64.b64decode(self._convert_to_bytes())
+            except binascii.Error:
+                try:
+                    self.state = base64.b64decode(self._convert_to_bytes() + b"=")
+                except binascii.Error:  # pragma: no cover
+                    self.state = base64.b64decode(self._convert_to_bytes() + b"==")
         return self
 
     @ChepyDecorators.call_stack
@@ -472,12 +501,11 @@ class DataFormat(ChepyCore):
         else:
             delimiters = [" ", "0x", "%", ",", ";", ":", r"\\n", "\\r\\n"]
             string = re.sub("|".join(delimiters), "", self.state)
-            # assert re.search(r"^[a-fA-F0-9]+$", string) is not None, "Invalid hex"
             self.state = string
             return self
 
     @ChepyDecorators.call_stack
-    def hexdump_to_str(self):
+    def str_from_hexdump(self):
         """Extract a string from a hexdump
         
         Returns:
@@ -487,6 +515,26 @@ class DataFormat(ChepyCore):
         self.state = "".join(re.findall(r"\|(.+)\|", self._convert_to_str()))
         return self
 
+    @ChepyDecorators.call_stack
+    def to_hexdump(self):
+        """Convert the state to hexdump
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        self.state = hexdump.hexdump(self._convert_to_bytes(), result="return")
+        return self
+
+    @ChepyDecorators.call_stack
+    def from_hexdump(self):
+        """Convert hexdump back to str
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        self.state = hexdump.restore(self._convert_to_str())
+        return self
+
     @ChepyDecorators.call_stack
     def url_encode(self, safe: str = ""):
         """URL encode
@@ -563,6 +611,16 @@ class DataFormat(ChepyCore):
         self.state = list(self._convert_to_str())
         return self
 
+    @ChepyDecorators.call_stack
+    def str_to_dict(self):
+        """Convert string to a dictionary
+        
+        Returns:
+            Chepy: The Chepy object.
+        """
+        self.state = yaml.safe_load(self._convert_to_str())
+        return self
+
     @ChepyDecorators.call_stack
     def to_charcode(self, escape_char: str = ""):
         """Convert a string to a list of unicode charcode
@@ -666,7 +724,11 @@ class DataFormat(ChepyCore):
                 "c",
             ]
         """
-        self.state = list(chr(int(s, 2)) for s in self.state)
+        if isinstance(self.state, list):
+            self.state = list(chr(int(s, 2)) for s in self.state)
+        else:
+            n = int(self._convert_to_str(), 2)
+            self.state = n.to_bytes((n.bit_length() + 7) // 8, "big")
         return self
 
     @ChepyDecorators.call_stack
