@@ -92,7 +92,7 @@ class Multimedia(ChepyCore):
                 "Valid resampling options are: nearest, antialias, bilinear, box and hamming"
             )
         image = Image.open(self._load_as_file())
-        resized = image.resize((width, height))
+        resized = image.resize((width, height), resample=resample)
         resized.save(fh, extension, quality=quality)
         self.state = fh.getvalue()
         return self
@@ -446,7 +446,7 @@ class Multimedia(ChepyCore):
         else:  # pragma: no cover
             new = image
 
-        formatted = new.save(fh, format_to)
+        new.save(fh, format_to)
         self.state = fh.getvalue()
         return self
 
@@ -485,6 +485,80 @@ class Multimedia(ChepyCore):
             new = image
 
         ImageDraw.Draw(new).text(coordinates, text, color)
-        formatted = new.save(fh, extension)
+        new.save(fh, extension)
         self.state = fh.getvalue()
         return self
+
+    @ChepyDecorators.call_stack
+    def lsb_dump_by_channel(self, channel: str = "r", column_first: bool = False): # pragma: no cover
+        """Dump LSB from a specific color channel
+        
+        Args:
+            channel (str, optional): Color channel. r, g, b. Defaults to 'r'.
+            column_first (bool, optional): Order by column first. Defaults to False.
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        channels = ["r", "g", "b"]
+        assert channel in channels, "Valid channels are {}".format("".join(channels))
+        if channel == "r":
+            index = 0
+        elif channel == "g":
+            index = 1
+        elif channel == "b":
+            index = 2
+        image = Image.open(self._load_as_file())
+        pixels = image.load()
+        if column_first:
+            height, width = image.size
+        else:
+            width, height = image.size
+
+        data = ""
+        for y in range(height):
+            for x in range(width):
+                try:
+                    pix = pixels[x, y][index]
+                    lsb = bin(pix).zfill(8)[-1]
+                    data += lsb
+                except IndexError:
+                    break
+
+        self.state = data
+        return self
+
+    @ChepyDecorators.call_stack
+    def msb_dump_by_channel(self, channel: str = "r", column_first: bool = False): # pragma: no cover
+        """Dump MSB from a specific color channel
+        
+        Args:
+            channel (str, optional): Color channel. r, g, b. Defaults to 'r'.
+            column_first (bool, optional): Order by column first. Defaults to False.
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        channels = ["r", "g", "b"]
+        assert channel in channels, "Valid channels are {}".format("".join(channels))
+        if channel == "r":
+            index = 0
+        elif channel == "g":
+            index = 1
+        elif channel == "b":
+            index = 2
+        image = Image.open(self._load_as_file())
+        pixels = image.load()
+        if column_first:
+            height, width = image.size
+        else:
+            width, height = image.size
+
+        data = ""
+        for x in range(height):
+            for y in range(width):
+                val = pixels[x, y][index]
+                data += ((bin(val)[2:]).zfill(8))[0]
+
+        self.state = data
+        return self
