@@ -1,9 +1,12 @@
 import difflib
 import regex as re
-from typing import Any
+import exrex
+from collections import OrderedDict
+from typing import Any, Union
 import pydash
 
 from ..core import ChepyCore, ChepyDecorators
+from .exceptions import StateNotDict, StateNotList
 import chepy.modules.internal.colors as _int_colors
 
 
@@ -89,7 +92,6 @@ class Utils(ChepyCore):
             remove.append("\t")
         if form_feeds:
             remove.append("\f")
-        print(remove)
         self.state = re.sub("|".join(remove), "", self._convert_to_str())
         return self
 
@@ -181,22 +183,23 @@ class Utils(ChepyCore):
         return self
 
     @ChepyDecorators.call_stack
-    def get_every_n(self, n: int):
-        """Get every nth item from a list or string. 
+    def select_every_n(self, n: int, start: int = 0):
+        """Select every nth item from a list or string. 
 
         Index starts at 0
         
         Args:
             n (int): n from 0
+            start (int): starting position. Defaults to 0.
         
         Returns:
             Chepy: The Chepy object. 
 
         Examples:
-            >>> Chepy(["a", 1, "lol", "b", True]).get_every_n(3)
+            >>> Chepy(["a", 1, "lol", "b", True]).select_every_n(3)
             ["a", "b"]
         """
-        self.state = self.state[0::n]
+        self.state = self.state[start::n]
         return self
 
     @ChepyDecorators.call_stack
@@ -204,62 +207,169 @@ class Utils(ChepyCore):
         """Get an array of unique list items
         
         Raises:
-            TypeError: If state is not a list
+            StateNotList: If state is not a list
         
         Returns:
             Chepy: The Chepy object.
         """
-        if isinstance(self.state, list):
-            self.state = pydash.uniq(self.state)
-            return self
-        else:  # pragma: no cover
-            raise TypeError("State is not a list")
+        assert isinstance(self.state, list), StateNotList()
+        self.state = pydash.uniq(self.state)
+        return self
 
     @ChepyDecorators.call_stack
-    def sorted(self, reverse: bool = False):
+    def sort_list(self, reverse: bool = False):
         """Sort a list
         
         Args:
             reverse (bool, optional): In reverse order. Defaults to False.
         
         Raises:
-            TypeError: If state is not list
+            StateNotList: If state is not list
         
         Returns:
             Chepy: The Chepy object.
 
         Examples:
-            >>> Chepy(["a", "b", "1", "2"]).sorted().o
+            >>> Chepy(["a", "b", "1", "2"]).sort_list().o
             ["1", "2", "a", "b"]
         """
-        if isinstance(self.state, (list)):
-            self.state = sorted(self.state)
-            return self
-        else:  # pragma: no cover
-            raise TypeError("State is not a list")
+        assert isinstance(self.state, list), StateNotList()
+        self.state = sorted(
+            self.state, key=lambda v: (isinstance(v, str), v), reverse=reverse
+        )
+        return self
 
     @ChepyDecorators.call_stack
-    def filter_by(self, predicate: Any = None):
-        """Filter a dict or list
+    def sort_dict_key(self, reverse: bool = False):
+        """Sort a dictionary by key
         
         Args:
-            predicate (Any, optional): What to filter by. Defaults to None.
+            reverse (bool, optional): Reverse sort order. Defaults to False.
         
-        Raises:
-            TypeError: If state is not a list or dict
+        Returns:
+            Chepy: The Chepy object. 
+
+        Examples:
+            >>> c = Chepy({'z': 'string', 'a': True, 'zz': 1, 'aaa': {'bb': 'data'}, 'ccc': [1,'a']})
+            >>> c.sort_dict_key(reverse=True)
+            {'zz': 1, 'z': 'string', 'ccc': [1, 'a'], 'aaa': {'bb': 'data'}, 'a': True}
+        """
+        assert isinstance(self.state, dict), StateNotDict()
+        self.state = dict(OrderedDict(sorted(self.state.items(), reverse=reverse)))
+        return self
+
+    @ChepyDecorators.call_stack
+    def sort_dict_value(self, reverse=False):
+        """Sort dictionary by value
+        
+        Args:
+            reverse (bool, optional): Reverse sort order. Defaults to False.
         
         Returns:
             Chepy: The Chepy object.
 
         Examples:
-            >>> Chepy('[{"a": 1}, {"b": 2}, {"a": 1, "b": 3}]').str_list_to_list().filter_by("b").o
+            >>> c = Chepy({'z': 'string', 'a': 'True', 'zz': '1', 'aaa': {'bb': 'data'}, 'ccc': [1,'a']})
+            >>> c.sort_dict_value()
+            {'zz': '1', 'a': 'True', 'ccc': [1, 'a'], 'z': 'string', 'aaa': {'bb': 'data'}}
+        """
+        assert isinstance(self.state, dict), StateNotDict()
+        self.state = dict(
+            OrderedDict(
+                sorted(self.state.items(), reverse=reverse, key=lambda x: str(x[1]))
+            )
+        )
+        return self
+
+    @ChepyDecorators.call_stack
+    def filter_list(self, by: Union[str, dict], regex: bool = True):
+        """Filter a list by a string regex or dict key
+        
+        Args:
+            by (Union[str, dict]): If string, supports regex. Or dictionary
+            regex (bool, optional): If pattern is a regex. Defaults to True
+        
+        Raises:
+            StateNotList: If state is not a list
+        
+        Returns:
+            Chepy: The Chepy object.
+
+        Examples:
+            >>> Chepy('[{"a": 1}, {"b": 2}, {"a": 1, "b": 3}]').str_list_to_list().filter_list("b").o
             [{"b": 2}, {"a": 1, "b": 3}]
         """
-        if isinstance(self.state, (list, dict)):
-            self.state = pydash.filter_(self.state, predicate)
-            return self
-        else:  # pragma: no cover
-            raise TypeError("State is not a list")
+        assert isinstance(self.state, list), StateNotList()
+        if regex:
+            pattern = by if isinstance(self.state[0], str) else by.encode()
+            self.state = [f for f in self.state if re.search(pattern, f)]
+        else:
+            self.state = pydash.filter_(self.state, by)
+        if len(self.state) == 1:
+            self.state = self.state[0]
+        return self
+
+    @ChepyDecorators.call_stack
+    def filter_list_by_length(self, length: int, exact: bool = False):
+        """Filter a list by length by specifying minimum length. 
+
+        It will also return items that exceed the specified length. 
+        
+        Args:
+            length (int): Minimum length to match
+            exact (bool): Match exact length
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        assert isinstance(self.state, list), StateNotList()
+        if exact:
+            self.state = [x for x in self.state if len(str(x)) == int(length)]
+        else:
+            self.state = [x for x in self.state if len(str(x)) >= int(length)]
+        return self
+
+    @ChepyDecorators.call_stack
+    def filter_dict_key(self, by: str):
+        """Filter dictionary by key
+        
+        Args:
+            by (str): Required. Key to filter by. 
+        
+        Returns:
+            Chepy: The Chepy object. 
+
+        Examples:
+            >>> Chepy({'some': 'dict', 'another': 'val'}).filter_dict_key('ano')
+            {'another': 'val'}
+        """
+        assert isinstance(self.state, dict), StateNotDict()
+        self.state = {
+            key: val for (key, val) in self.state.items() if re.search(by, str(key))
+        }
+        return self
+
+    @ChepyDecorators.call_stack
+    def filter_dict_value(self, by: str):
+        """Filter dictionary by value. 
+
+        This method does descend into nested dictionary values. 
+        
+        Args:
+            by (str): Required. Value to filter by. 
+        
+        Returns:
+            Chepy: The Chepy object. 
+
+        Examples:
+            >>> Chepy({'some': 'dict', 'another': 'val'}).filter_dict_value('val')
+            {'another': 'val'}
+        """
+        assert isinstance(self.state, dict), StateNotDict()
+        self.state = {
+            key: val for (key, val) in self.state.items() if re.search(by, str(val))
+        }
+        return self
 
     @ChepyDecorators.call_stack
     def slice(self, start: int = 0, end: int = None):
@@ -307,10 +417,6 @@ class Utils(ChepyCore):
         
         Returns:
             Chepy: The Chepy object.
-
-        Examples:
-            >>> Chepy("\\$ome' d@ta").unescape_string().o
-            "$ome' d@ta"
         """
         self.state = re.escape(self._convert_to_str())
         return self
@@ -321,6 +427,10 @@ class Utils(ChepyCore):
         
         Returns:
             Chepy: The Chepy object.
+
+        Examples:
+            >>> Chepy("\\$ome' d@ta").unescape_string().o
+            "$ome' d@ta"
         """
         self.state = re.sub(r"\\", "", self._convert_to_str())
         return self
@@ -385,22 +495,21 @@ class Utils(ChepyCore):
         def process_tag(tag, i1, i2, j1, j2):  # pragma: no cover
             if tag == "replace":
                 if colors:
-                    return _int_colors.BLUE(matcher.b[j1:j2])
+                    return _int_colors.blue(matcher.b[j1:j2])
                 else:
                     return "{" + matcher.a[i1:i2] + "->" + matcher.b[j1:j2] + "}"
             if tag == "delete":
                 if colors:
-                    return _int_colors.RED(matcher.a[i1:i2])
+                    return _int_colors.red(matcher.a[i1:i2])
                 else:
                     return "{-" + matcher.a[i1:i2] + "}"
             if tag == "equal":
                 return matcher.a[i1:i2]
             if tag == "insert":
                 if colors:
-                    return _int_colors.GREEN(matcher.b[j1:j2])
+                    return _int_colors.green(matcher.b[j1:j2])
                 else:
                     return "{+" + matcher.b[j1:j2] + "}"
-                return "{+" + matcher.b[j1:j2] + "}"
             assert False, "Unknown tag %r" % tag
 
         self.state = "".join(process_tag(*t) for t in matcher.get_opcodes())
@@ -462,3 +571,19 @@ class Utils(ChepyCore):
         self.state = list(set(self.state))
         return self
 
+    @ChepyDecorators.call_stack
+    def regex_to_str(self, all_combo: bool = False):
+        """Convert a regex to a matching string
+        
+        Args:
+            all_combo (bool, optional): Generate all combos that match regex. Defaults to False.
+        
+        Returns:
+            Chepy: The Chepy object. 
+        """
+        if all_combo:
+            self.state = list(exrex.generate(self._convert_to_str()))
+        else:
+            self.state = exrex.getone(self._convert_to_str())
+        return self
+
